﻿using System;
using System.Collections.Generic;

namespace TemplatePattern
{
    public abstract class ShippingCalculatorBase
    {
        public decimal CalculateShippingRate( string zipCode, double itemWeight, Size boxSize )
        {
            ZipCode = zipCode;
            DetermineWeightMultiplier( itemWeight );
            DetermineLocationSurcharge( zipCode );
            DetermineBoxSizeMultipler( boxSize );

            return (decimal) ( ( WeightMultipler * BoxMultipler ) + LocationSurcharge );
        }

        

        protected abstract void DetermineLocationSurcharge( string zipCode );

        protected abstract void DetermineWeightMultiplier( double weight );

        protected abstract void DetermineBoxSizeMultipler( Size boxSize );

        protected string ZipCode { get; set; }
        protected double LocationSurcharge { get; set; }
        protected Int32 WeightMultipler { get; set; }
        protected double BoxMultipler { get; set; }
    }

    public class GroundShippingCalculator : ShippingCalculatorBase {
        
        private Dictionary<string, double> _locationSurcharge = new Dictionary<string, double>();
        public GroundShippingCalculator()
        {
            _locationSurcharge.Add( "14410", 1.0 );
            _locationSurcharge.Add( "55555", 1.35 );
            _locationSurcharge.Add( "90120", 1.50 );
        }

        protected override void DetermineLocationSurcharge( string zipCode )
        {
            if ( _locationSurcharge.ContainsKey( zipCode ) )
            {
                LocationSurcharge = _locationSurcharge[zipCode];
            }
            else
            {
                LocationSurcharge = 1;
            }
        }

        protected override void DetermineWeightMultiplier( double weight )
        {
            if (weight <= 10)
            {
                WeightMultipler = 1;
            }
            else if ( weight <= 25 )
            {
                WeightMultipler = 2;
            }
            else
            {
                WeightMultipler = 3;
            }
        }

        protected override void DetermineBoxSizeMultipler( Size boxSize )
        {
            int totalVolumne = boxSize.GetTotalVolumne();

            if (totalVolumne <= 250  )
            {
                BoxMultipler = 1;
            }
            else
            {
                BoxMultipler = 1.5;
            }
        }
    }

    public class AirShippingCalculator : ShippingCalculatorBase
    {

        private Dictionary<string, double> _locationSurcharge = new Dictionary<string, double>();
        public AirShippingCalculator()
        {
            _locationSurcharge.Add( "14410", 3.0 );
            _locationSurcharge.Add( "55555", 1.35 );
            _locationSurcharge.Add( "90120", .25 );
        }

        protected override void DetermineLocationSurcharge( string zipCode )
        {
            if ( _locationSurcharge.ContainsKey( zipCode ) )
            {
                LocationSurcharge = _locationSurcharge[ zipCode ];
            }
            else
            {
                LocationSurcharge = 1;
            }
        }

        protected override void DetermineWeightMultiplier( double weight )
        {
            if ( weight <= 10 )
            {
                WeightMultipler = 1;
            }
            else if ( weight <= 25 )
            {
                WeightMultipler = 3;
            }
            else
            {
                WeightMultipler = 7;
            }
        }

        protected override void DetermineBoxSizeMultipler( Size boxSize )
        {
            int totalVolumne = boxSize.GetTotalVolumne();

            if ( totalVolumne <= 250 )
            {
                BoxMultipler = 1;
            }
            else if (totalVolumne <= 500 )
            {
                BoxMultipler = 3;
            }
            else
            {
                BoxMultipler = 5;
            }
        }
    }

    public class Size
    {
        public Size( int width, int height, int length )
        {
            Width = width;
            Height = height;
            Length = length;
        }

        public Int32 Width { get; private set; }
        public Int32 Height { get; private set; }
        public Int32 Length { get; private set;}

        public Int32 GetTotalVolumne()
        {
            return Width * Height * Length;
        }

    }
}

